@@ -7,6 +7,7 @@ import csv
 import pytz
 import tempfile
 from functools import reduce
+import sys
 
 class StatusTypes:
     open = 'open'
@@ -81,7 +82,7 @@ class CycleTimeQueries(QueryManager):
 
         super(CycleTimeQueries, self).__init__(jira, **settings)
 
-    def cycle_data(self, verbose=False, result_cycle=None, result_size=None):
+    def cycle_data(self, verbose=False, result_cycle=None, result_size=None, result_edges=None):
         """Get data from JIRA for cycle/flow times and story points size change.
 
         Build a numerically indexed data frame with the following 'fixed'
@@ -126,7 +127,19 @@ class CycleTimeQueries(QueryManager):
             'completed_timestamp': {'data': [], 'dtype': 'datetime64[ns]'}
         }
 
-        buffer = tempfile.SpooledTemporaryFile(max_size=20000,mode='w+t')
+        if sys.platform.startswith('win'):
+            buffer = open("cycledata.tmp", "w+",1)
+            # Opens a file for writing only in binary format. Overwrites the file if the file exists.
+            # buffering value is 1
+            # Windows users seem to have a problem with spooled file
+        else:
+            buffer = tempfile.SpooledTemporaryFile(max_size=20000, mode='w+t')
+
+        #issuelinks = open("issuelinks.csv", "w+", 1)
+        #df_edges = pd.DataFrame()
+        #df_edges = pd.DataFrame(columns=['Source', 'OutwardLink', 'Target', 'Inwardlink','LinkType'])
+        #df_edges.to_csv(issuelinks, columns=['Source', 'OutwardLink', 'Target', 'Inwardlink','LinkType'], header=True, index=None, sep='\t',encoding='utf-8')
+
         df_size_history = pd.DataFrame( columns=['key','fromDate','toDate','size'])
         df_size_history.to_csv(buffer, columns=['key', 'fromDate', 'toDate', 'size'], header=True, index=None, sep='\t',encoding='utf-8')
 
@@ -142,16 +155,31 @@ class CycleTimeQueries(QueryManager):
         for criteria in self.settings['queries']:
             for issue in self.find_issues(criteria, order='updatedDate DESC', verbose=verbose):
 
-                item = {
-                    'key': issue.key,
-                    'url': "%s/browse/%s" % (self.jira._options['server'], issue.key,),
-                    'issue_type': issue.fields.issuetype.name,
-                    'summary': issue.fields.summary.encode('utf-8'),
-                    'status': issue.fields.status.name,
-                    'resolution': issue.fields.resolution.name if issue.fields.resolution else None,
-                    'cycle_time': None,
-                    'completed_timestamp': None
-                }
+                # Deal with the differences in strings between Python 2 & 3
+                if (sys.version_info > (3, 0)):
+                    # Python 3 code in this block
+                    item = {
+                        'key': issue.key,
+                        'url': "%s/browse/%s" % (self.jira._options['server'], issue.key,),
+                        'issue_type': issue.fields.issuetype.name,
+                        'summary': issue.fields.summary,  # .encode('utf-8'),
+                        'status': issue.fields.status.name,
+                        'resolution': issue.fields.resolution.name if issue.fields.resolution else None,
+                        'cycle_time': None,
+                        'completed_timestamp': None
+                    }
+                else:
+                    # Python 2 code in this block
+                    item = {
+                        'key': issue.key,
+                        'url': "%s/browse/%s" % (self.jira._options['server'], issue.key,),
+                        'issue_type': issue.fields.issuetype.name,
+                        'summary': issue.fields.summary.encode('utf-8'),
+                        'status': issue.fields.status.name,
+                        'resolution': issue.fields.resolution.name if issue.fields.resolution else None,
+                        'cycle_time': None,
+                        'completed_timestamp': None
+                    }
 
                 for name, field_name in self.fields.items():
                     item[name] = self.resolve_field_value(issue, name, field_name)
@@ -162,7 +190,43 @@ class CycleTimeQueries(QueryManager):
                 for cycle_name in cycle_names:
                     item[cycle_name] = None
 
-                # Get Story Points size changes history
+                # Get the relationships for this issue
+                edges = []  # Source, Target, Inward Link, Outward Link, Type
+                issuelinks = issue.fields.issuelinks
+
+                # It is seems that having an Epic Parent does not record an Epic Link, just the name "Epic Name"
+                # Creating Epic relationship requires more work. Also each Jira instance will have different customfields for Epic data
+                # Remove this code.
+                #issueEpic = issue.fields.customfield_10008 if issue.fields.customfield_10008 else None  # Epic Link
+                #if issueEpic is not None:
+                #    data = {'Source':issueEpic, 'Target':issue.key, 'InwardLink':'Belongs to Epic', 'OutwardLink':'Issue in Epic', 'LinkType':'EpicIssue'}
+                #    edges.append(data)
+
+                for link in issuelinks:
+                    inwardissue = None
+                    outwardissue = None
+                    try:
+                        inwardissue = link.inwardIssue.key
+                    except:
+                        outwardissue = link.outwardIssue.key
+
+                    if inwardissue is not None:
+                        data = {'Source':inwardissue, 'Target':issue.key, 'InwardLink':link.type.inward, 'OutwardLink': link.type.outward, 'LinkType':link.type.name}
+                    else:
+                        data = {'Source':issue.key, 'Target': outwardissue, 'InwardLink':link.type.inward, 'OutwardLink':link.type.outward, 'LinkType':link.type.name}
+                    edges.append(data)
+
+                if len(edges)>0:
+                    try:
+                        df_edges
+                    except NameError:
+                        #print('Not found')
+                        df_edges = pd.DataFrame(edges)
+                    else:
+                        df_links = pd.DataFrame(edges)
+                        df_edges=df_edges.append(df_links)  # = pd.DataFrame(edges)
+                # Got all the relationships for this iisue
+
                 rows = []
                 for snapshot in self.iter_size_changes(issue):
                     data= {'key':snapshot.key,'fromDate':snapshot.date,'size':snapshot.size}
@@ -175,13 +239,21 @@ class CycleTimeQueries(QueryManager):
                 # Round Down datetimes to full dates
                 df['fromDate'] = df['fromDate'].apply(lambda dt: datetime.datetime(dt.year, dt.month, dt.day))
                 df['toDate'] = df['toDate'].apply(lambda dt: datetime.datetime(dt.year, dt.month, dt.day))
+                # If we only have one row of size changes and current issue has a size then it must have been created with a size value at creation.
+                # This size will not be recorded in the size_change record.
+                # Hence update the single row we have with the current issue size.
+                # Get Story Points size changes history
+                #If condition is met update the size cell
+                if (item['StoryPoints'] is not None ) and (len(df)==1):
+                    df.loc[df.index[0], 'size'] = item['StoryPoints']
+
                 # Append to csv file
                 df.to_csv(buffer, columns=['key', 'fromDate', 'toDate', 'size'], header=None,
                            mode='a', sep='\t', date_format='%Y-%m-%d',encoding='utf-8')
                 #print(rows)
 
                 # Record date of status changes
-                for snapshot in self.iter_changes(issue, False):
+                for snapshot in self.iter_changes(issue, True):
                     snapshot_cycle_step = self.settings['cycle_lookup'].get(snapshot.status.lower(), None)
                     if snapshot_cycle_step is None:
                         if verbose:
@@ -242,11 +314,26 @@ class CycleTimeQueries(QueryManager):
         result_size = pd.DataFrame()
         buffer.seek(0)
         result_size = result_size.from_csv(buffer, sep='\t')
+        buffer.close()
+
+        try:
+            df_edges
+        except NameError:
+            # print('Not found')
+            df_edges = pd.DataFrame()
+        try:
+            df_edges = df_edges[['Source', 'OutwardLink', 'Target', 'InwardLink','LinkType']] # Specify dataframe sort order
+            #df_edges.to_csv("myedges.csv", sep='\t', index=False,encoding='utf-8')
+        except KeyError:
+            print('We had a key error in pandas. No issue edges found.')
+
+        result_edges=df_edges
+
         result_size.set_index('key')
         result_size['toDate'] = pd.to_datetime(result_size['toDate'], format=('%Y-%m-%d'))
         result_size['fromDate'] = pd.to_datetime(result_size['fromDate'], format=('%Y-%m-%d'))
 
-        return result_cycle, result_size
+        return result_cycle, result_size, result_edges
 
     def size_history(self,size_data):
         """Return the a DataFrame,
@@ -345,6 +432,33 @@ class CycleTimeQueries(QueryManager):
             sum_row = df_result[df.columns].sum()  # Sum Columns
             return pd.DataFrame(data=sum_row).T  # Transpose into row dataframe and return
 
+        # Helper function to return the right most cells in 2D array
+        def keeprightmoststate(df):
+            """
+            Incoming matrix columns represents items in workflow states
+            States progress from left to right.
+            We what to zero out items, other than right most value.
+            :param df:
+            :return: pandas dataframe row with sum of column items
+            """
+
+            def last_number(lst):
+                if all(map(lambda x: x == 0, lst)):
+                    return 0
+                elif lst[-1] != 0:
+                    return len(lst) - 1
+                else:
+                    return last_number(lst[:-1])
+
+            def fill_others(lst):
+                new_lst = [0] * len(lst)
+                new_lst[last_number(lst)] = lst[last_number(lst)]
+                return new_lst
+
+            df_zeroed = df.fillna(value=0)  # ,inplace = True   Get rid of non numeric items. Make a ?deep? copy
+            df_result = df_zeroed.apply(lambda x: fill_others(x.values.tolist()), axis=1)
+            return df_result
+
         # Define helper function
         def hide_greater_than_date(cell, adate):
             """ Helper function to compare date values in cells
@@ -358,6 +472,21 @@ class CycleTimeQueries(QueryManager):
                 return True
             return False  # We have a date value in cell and it is less than or equal to input date
 
+        # Helper function
+        def appendDFToCSV(df, csvFilePath, sep="\t",date_format='%Y-%m-%d', encoding='utf-8'):
+            import os
+            if not os.path.isfile(csvFilePath):
+                df.to_csv(csvFilePath, mode='a', index=False, sep=sep, date_format=date_format, encoding=encoding)
+            elif len(df.columns) != len(pd.read_csv(csvFilePath, nrows=1, sep=sep).columns):
+                raise Exception(
+                    "Columns do not match!! Dataframe has " + str(len(df.columns)) + " columns. CSV file has " + str(
+                        len(pd.read_csv(csvFilePath, nrows=1, sep=sep).columns)) + " columns.")
+            elif not (df.columns == pd.read_csv(csvFilePath, nrows=1, sep=sep).columns).all():
+                raise Exception("Columns and column order of dataframe and csv file do not match!!")
+            else:
+                df.to_csv(csvFilePath, mode='a', index=False, sep=sep, header=False, date_format=date_format, encoding=encoding)
+
+
         #print(pointscolumn)
 
         # List of all state change columns that may have date value in them
@@ -424,33 +553,14 @@ class CycleTimeQueries(QueryManager):
                 # Apply function to each cell and only make it visible if issue was in state on or after the filter date
                 df_filtered = df.applymap(lambda x: 0 if hide_greater_than_date(x, filterdate) else 1)
 
-                if pointscolumn:
-                    # Function to create column of sizes at a given point in time
-                    # Needs mydate and size_data
-                    # def lookup_size(x):
-                    #     # needs datevar and df_size_history
-                    #     size_data_for_key = size_data[size_data['key']==x]
-                    #
-                    #     try:
-                    #         df_filtered = size_data_for_key[size_data_for_key.apply(
-                    #             lambda row: datetime.date(row.fromDate.year, row.fromDate.month,
-                    #                                   row.fromDate.day) <= filterdate <= datetime.date(row.toDate.year,
-                    #                                                                                    row.toDate.month,
-                    #                                                                                    row.toDate.day),
-                    #             axis=1)]
-                    #     except:
-                    #         df_filtered = None
-                    #
-                    #     #df_filtered = size_data[
-                    #     #    size_data.apply(lambda row: datetime.date(row.fromDate.year, row.fromDate.month, row.fromDate.day) <= filterdate <= datetime.date(row.toDate.year, row.toDate.month, row.toDate.day), axis=1)]
-                    #     if len(df_filtered):
-                    #         key_size_value = df_filtered.iloc[-1]['size']
-                    #     else:
-                    #         key_size_value = None
-                    #     #print(key_size_value)
-                    #     return key_size_value
+                if stacked:
+                    df_filtered=keeprightmoststate(df_filtered)
 
-                    #df_size_on_day = cycle_data['key'].apply(lambda row: lookup_size(row))
+                if pointscolumn:
+
+                    # For debug
+                    #if filterdate.isoformat() == '2016-11-22':
+                    #    size_history.loc[filterdate.isoformat()].to_csv("debug-size-history.csv")
                     storypoints_series_on = size_history.loc[filterdate.isoformat()].T
                     df_size_on_day = pd.Series.to_frame(storypoints_series_on)
                     df_size_on_day.columns = [pointscolumn]
@@ -460,6 +570,13 @@ class CycleTimeQueries(QueryManager):
                     right = df_size_on_day
                     result = left.join(right, on=['key'])  # http://pandas.pydata.org/pandas-docs/stable/merging.html\
                     df_countable = pd.concat([result, df_filtered], axis=1)
+                    # for debuging and analytics append the days state to file
+                    df_countable['date'] = filterdate.isoformat()
+                    if stacked:
+                        file_name = "daily-cfd-stacked-run-at"+ timenowstr + ".csv"
+                    else:
+                        file_name = "daily-cfd-run-at" + timenowstr + ".csv"
+                    appendDFToCSV(df_countable, file_name )
                 else:
                     df_countable = df_filtered
 
@@ -468,17 +585,11 @@ class CycleTimeQueries(QueryManager):
                 if pointscolumn:
                     for size_state in self.settings['sized_statuses']: #states_to_size:
                         sizedStateName = size_state + 'Sized'
-                        df_countable[sizedStateName] = df_countable.apply(
-                            lambda row: (row[pointscolumn] * row[size_state] if row[pointscolumn] > 0 else (1.0 * row[size_state])), axis=1)
-
-                # Slice out the columns we want for CFD
-                # df_slice= df_countable.loc[:,('Open','Analysis','PrioritizedSized','In ProcessSized','DoneSized')]
-                #print(slice_columns)
-                #df_slice = df_countable.loc[:, ('Open', 'AnalysisSized', 'CommittedSized', 'DevelopSized', 'DoneSized')]
+                        df_countable[sizedStateName] = df_countable.apply( lambda row: (row[pointscolumn] * row[size_state] ), axis=1)
 
                 # For debugging write dataframe to sheet for current day.
                 #file_name="countable-cfd-for-day-"+ filterdate.isoformat()+timenowstr+".csv"
-                #df_countable.to_csv(file_name, sep='\t', lineterminator='\n', encoding='utf-8', quoting=csv.QUOTE_ALL)
+                #df_countable.to_csv(file_name, sep='\t', encoding='utf-8', quoting=csv.QUOTE_ALL)
 
                 df_slice = df_countable.loc[:,slice_columns].copy()
                 df_sub_sum = cumulativeColumnStates(df_slice,stacked)
@@ -606,7 +717,8 @@ class CycleTimeQueries(QueryManager):
 
             while current_value < target_value:
                 current_date += frequency
-                current_value += get_sample()
+                sample_data = get_sample()
+                current_value += sample_data
 
                 dates.append(current_date)
                 steps.append(current_value)
@@ -625,10 +737,16 @@ class CycleTimeQueries(QueryManager):
         percentiles=[0.5, 0.75, 0.85, 0.95],
         sized = ''
     ):
-        if len(cfd_data.index) == 0:
-            raise Exception("Cannot calculate burnup forecast with no data")
-        if len(throughput_data.index) == 0:
-            raise Exception("Cannot calculate burnup forecast with no completed items")
+        try:
+            if len(cfd_data.index) == 0:
+                raise Exception("Cannot calculate burnup forecast with no data")
+
+            if len(throughput_data.index) == 0:
+                raise Exception("Cannot calculate burnup forecast with no completed items")
+
+        except AttributeError:
+            # Method does not exist.  What now?
+            raise Exception("Cannot calculate burnup forecast with data or no completed items")
 
         # Debug - what are the column names
         #print("backlog_column --> {}  done_column --> {}".format(backlog_column, done_column))
